/************************************************************************
	game.c

    Simon18F - An MB Electronics Simon clone based on the PIC18F2550
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

// Global includes
#include <htc.h>

// Local includes
#include "hardware.h"
#include "interrupt.h"
#include "sound.h"
#include "delay.h"
#include "lights.h"
#include "game.h"

// This is the main game loop which is the first procedure called
// when the game is turned on.  Here we monitor the start, last
// and longest buttons.
void mainGameLoop(void)
{
	int counter;

	// Has start been pressed?
	if (hardware.startButton.state == ON)
	{
		// Note: the game.restartFlag is set if the player presses start
		//       during a game.  This causes the game to gameOver and then
		//       return, we have to pick up the flag or the player would
		//       have to press start again to restart the game.
		do
		{
			game.restartFlag = FALSE;

			// Read the game number slider
			game.gameNumber = hardware.gameSwitch.position;
	
			// Read the skill level slider
			game.skillLevel = hardware.skillSwitch.position;

			// Start the selected game
			simonSays(game.gameNumber);
		} while (game.restartFlag);

		// Reset the inactivity timer
		hardware.delayCounter = 0;
	}
	
	// Has last been pressed?
	if (hardware.lastButton.state == ON)
	{
		// If there is a last sequence play it back
		if (game.currentSequencePointer != -1)
		{
			// Play back the last sequence
			for (counter = 0; counter <= game.currentSequencePointer; counter++)
			{
				colourOn(game.currentSequence[counter]);

				gameSound(game.currentSequence[counter]);
				delayMilliSeconds(PLAYBACK_SLOW);
	
				colourOff(game.currentSequence[counter]);
				soundOff();
				delayMilliSeconds(50);
			}
		}

		delayMilliSeconds(800);

		// Reset the inactivity timer
		hardware.delayCounter = 0;
	}

	// Has longest been pressed?
	if (hardware.longestButton.state == ON)
	{
		// If there is a longest sequence play it back
		if (game.longestSequenceLength != -1)
		{
			// Play back the last sequence
			for (counter = 0; counter <= game.longestSequenceLength; counter++)
			{
				colourOn(game.longestSequence[counter]);
				gameSound(game.longestSequence[counter]);
				delayMilliSeconds(PLAYBACK_SLOW);
	
				colourOff(game.longestSequence[counter]);
				soundOff();
				delayMilliSeconds(50);
			}
		}

		delayMilliSeconds(800);

		// Reset the inactivity timer
		hardware.delayCounter = 0;
	}

	// Has red been pressed? (Volume up)
	if (hardware.colourButton[RED].state == ON)
	{
		hardware.volumeLevel += 10;
		if (hardware.volumeLevel > 200) hardware.volumeLevel = 200;
		else
		{
			colourOn(RED);
			soundOn(RED_SOUND);
			delayMilliSeconds(20);
			soundOff();
			delayMilliSeconds(250);
			colourOff(RED);
		}
	}

	// Has blue been pressed? (Volume down)
	if (hardware.colourButton[BLUE].state == ON)
	{
		hardware.volumeLevel -= 10;
		if (hardware.volumeLevel < 40) hardware.volumeLevel = 40;
		else
		{
			colourOn(BLUE);
			soundOn(BLUE_SOUND);
			delayMilliSeconds(20);
			soundOff();
			delayMilliSeconds(250);
			colourOff(BLUE);
		}
	}
}

// Game 1: Simon Says
// This is an emulation of the original Simon game 1; Simon plays a sequence
// of colours and the player must repeat the sequence. With every successful
// repetition Simon adds another colour until the sequence length matches
// the skill level's maximum.

// Game 2: Player Says
// This is an emulation of the original Simon game 2;  This game is for 2 or more
// players.  Simon shows an initial colour and the 1st player must repeat the colour
// and add one more, then the 2nd player repeats both colours and so on until the 
// sequence maximum is met.
//
// Game 3: Choose your colour
// This is an emulation of the original Simon game 3;  This game is for 2-4 players.
// The game is the same as game 1 except each player owns one or two colours.  If the
// colour gets 'razzed' the colour is out and a new sequence begins without the colour.
// Game play continues until 31 colours are sequenced or only one 'winning' colour
// remains in the game.
//
// Note: In the original version of Simon the manual instructed the player to always
// select skill level 4 for games 2 and 3 (otherwise the maximum sequence length will be
// lower than the maximum).  This 'feature' is recreated here and the skill level
// setting has the same effect as in game 1.
//
void simonSays(unsigned char gameNumber)
{
	unsigned char gameOver = FALSE;			// The game over flag
	unsigned char waitingForInput = TRUE;	// Flag is true when waiting for user input
	unsigned char colourCount;				// Used to count the number of active colours left

	int currentColour;						// Use to store the currently pressed colour button	
	int loop; 								// General purpose 'for loop' variable
	int playBackDelay;						// Used to store the playback delay when repeating the sequence

	// Reset the active colours
	game.colourActive[GREEN]  = TRUE;
	game.colourActive[RED]    = TRUE;
	game.colourActive[YELLOW] = TRUE;
	game.colourActive[BLUE]   = TRUE;	

	// Reset the sequence and player positions
	game.currentSequencePointer = -1;
	game.playerSequencePointer  = 0;

	while (!gameOver)
	{
		if ((gameNumber == 2 && game.currentSequencePointer == -1) ||
			gameNumber == 1 || gameNumber == 3)
		{
			// Add one to the sequence position
			game.currentSequencePointer++;

			// Add a colour to the sequence from the active colours
			do game.currentSequence[game.currentSequencePointer] = game.randomNumber;
			while (game.colourActive[game.currentSequence[game.currentSequencePointer]] != TRUE);
	
			// Replay current sequence
			if (game.currentSequencePointer < 5) playBackDelay = PLAYBACK_SLOW;
			else if (game.currentSequencePointer < 13) playBackDelay = PLAYBACK_MEDIUM;
			else playBackDelay = PLAYBACK_FAST;

			for (loop = 0; loop <= game.currentSequencePointer; loop++)
			{
				colourOn(game.currentSequence[loop]);
				gameSound(game.currentSequence[loop]);
				delayMilliSeconds(playBackDelay);
				
				colourOff(game.currentSequence[loop]);
				soundOff();
				delayMilliSeconds(50);
			}
		}
	
		// Reset the button inactivity timer
		hardware.delayCounter = 0;

		// Waiting for input = TRUE
		waitingForInput = TRUE;

		while(waitingForInput)
		{
			// Start button pressed?
			if (hardware.startButton.state == ON)
			{
				// Current sequence longer than longest?
				if (game.currentSequencePointer > game.longestSequenceLength)
				{
					// Store the current sequence as longest
					for (loop = 0; loop <= game.currentSequencePointer; loop++)
						game.longestSequence[loop] = game.currentSequence[loop];
	
					game.longestSequenceLength = game.currentSequencePointer;
				}
				
				// Game over (due to start button being pressed)
				gameOver = TRUE;

				waitingForInput = FALSE;
			}
			else
			{
				// Colour button pressed?
				currentColour = -1;
				if (hardware.colourButton[GREEN].state == ON) 	currentColour = GREEN;
				if (hardware.colourButton[RED].state == ON) 	currentColour = RED;
				if (hardware.colourButton[YELLOW].state == ON) 	currentColour = YELLOW;
				if (hardware.colourButton[BLUE].state == ON)	currentColour = BLUE;

				if (currentColour != -1)
				{
					waitingForInput = FALSE;

					// Is playerPosition > sequence length?
					if (game.playerSequencePointer > game.currentSequencePointer)
					{
						game.currentSequencePointer++;

						// Add the colour to the sequence
						game.currentSequence[game.currentSequencePointer] = currentColour;

						// Light the depressed colour button
						colourOn(currentColour);

						// Play the colour's tone
						gameSound(currentColour);

						// Button released?
						while (hardware.colourButton[currentColour].state == ON);

						// Turn off the colour's light
						colourOff(currentColour);

						// Turn off the colour's sound
						soundOff();

						game.playerSequencePointer = 0;
					}
					else
					{
						// Does the button match the sequence?
						if (currentColour == game.currentSequence[game.playerSequencePointer])
						{
							// Light the depressed colour button
							colourOn(currentColour);

							// Play the colour's tone
							gameSound(currentColour);

							// Button released?
							while (hardware.colourButton[currentColour].state == ON);

							// Turn off the colour's light
							colourOff(currentColour);

							// Turn off the colour's sound
							soundOff();

							// End of sequence?
							if (game.playerSequencePointer < game.currentSequencePointer)
							{
								// Reset the button inactivity timer
								hardware.delayCounter = 0;
			
								// Add 1 to player position
								game.playerSequencePointer++;

								// Waiting for input = TRUE
								waitingForInput = TRUE;
							}
							else
							{
								// Sequence length maximum for skill level?
								if ((game.skillLevel == 1 && game.currentSequencePointer ==  7) ||
									(game.skillLevel == 2 && game.currentSequencePointer == 13) ||
									(game.skillLevel == 3 && game.currentSequencePointer == 19) ||
									(game.skillLevel == 4 && game.currentSequencePointer == 30))
								{
									if (gameNumber == 3 && game.skillLevel == 4)
									{
										// Play special victory tone
										specialVictoryRazz();
									}
									else
									{
										// Play victory razz
										victoryRazz(currentColour);
									}

									// Current Sequence longer than longest?
									if (game.currentSequencePointer > game.longestSequenceLength)
									{
										// Store the current sequence as longest
										for (loop = 0; loop <= game.currentSequencePointer; loop++)
											game.longestSequence[loop] = game.currentSequence[loop];
						
										game.longestSequenceLength = game.currentSequencePointer;
									}

									gameOver = TRUE;
								}
								else
								{
									// Game number?
									if (gameNumber == 1 || gameNumber == 3)
									{
										// Set player position to 0
										game.playerSequencePointer = 0;
									}
									else
									{
										// Is player position > sequence length?
										if (game.playerSequencePointer > game.currentSequencePointer)
										{
											// Set player position to 0
											game.playerSequencePointer = 0;
										}
										else
										{
											// Add 1 to player position
											game.playerSequencePointer++;
										}
									}
								}
							}
						}
						else
						{
							// Play looser alert
							colourOn(currentColour);
							gameSound(FAIL);
							delayMilliSeconds(1500);
							colourOff(currentColour);
							soundOff();

							// Current sequence longer than longest?
							if (game.currentSequencePointer > game.longestSequenceLength)
							{
								// Store the current sequence as longest
								for (loop = 0; loop <= game.currentSequencePointer; loop++)
									game.longestSequence[loop] = game.currentSequence[loop];
				
								game.longestSequenceLength = game.currentSequencePointer;
							}

							// Game number
							if (gameNumber == 1 || gameNumber == 2)
							{
								gameOver = TRUE;
							}
							else
							{
								// Eliminate colour which was depressed
								game.colourActive[currentColour] = FALSE;

								// More than one colour left?
								colourCount = 0;
								if (game.colourActive[GREEN] == TRUE) 	colourCount++;
								if (game.colourActive[RED] == TRUE) 	colourCount++;
								if (game.colourActive[YELLOW] == TRUE) 	colourCount++;
								if (game.colourActive[BLUE] == TRUE) 	colourCount++;

								if (colourCount > 1)
								{
									// Set sequence position to -1
									game.currentSequencePointer = -1;

									// Set player position to 0
									game.playerSequencePointer = 0;
								}
								else
								{
									// Play victory tone
									if (game.colourActive[GREEN] == TRUE) 	victoryRazz(GREEN);
									if (game.colourActive[RED] == TRUE) 	victoryRazz(RED);
									if (game.colourActive[YELLOW] == TRUE) 	victoryRazz(YELLOW);
									if (game.colourActive[BLUE] == TRUE) 	victoryRazz(BLUE);

									gameOver = TRUE;
								}
							}
						}
					}
				}
				else
				{
					// > 3 seconds inactivity?
					if (hardware.delayCounter > 48000)
					{
						// Waiting for input = FALSE
						waitingForInput = FALSE;

						// Play looser alert
						colourOn(game.currentSequence[game.playerSequencePointer]);
						gameSound(FAIL);
						delayMilliSeconds(1500);
						colourOff(game.currentSequence[game.playerSequencePointer]);
						soundOff();

						// Current sequence longer than longest?
						if (game.currentSequencePointer > game.longestSequenceLength)
						{
							// Store the current sequence as longest
							for (loop = 0; loop <= game.currentSequencePointer; loop++)
								game.longestSequence[loop] = game.currentSequence[loop];
			
							game.longestSequenceLength = game.currentSequencePointer;
						}

						// Game number
						if (gameNumber == 1 || gameNumber == 2)
						{
							gameOver = TRUE;
						}
						else
						{
							// Eliminate colour which was missed
							game.colourActive[game.currentSequence[game.playerSequencePointer]] = FALSE;

							// More than one colour left?
							colourCount = 0;
							if (game.colourActive[GREEN] == TRUE) 	colourCount++;
							if (game.colourActive[RED] == TRUE) 	colourCount++;
							if (game.colourActive[YELLOW] == TRUE) 	colourCount++;
							if (game.colourActive[BLUE] == TRUE) 	colourCount++;

							if (colourCount > 1)
							{
								// Set sequence position to -1
								game.currentSequencePointer = -1;

								// Set player position = 0;
								game.playerSequencePointer = 0;
							}
							else
							{
								// Play victory tone
								if (game.colourActive[GREEN] == TRUE) 	victoryRazz(GREEN);
								if (game.colourActive[RED] == TRUE) 	victoryRazz(RED);
								if (game.colourActive[YELLOW] == TRUE) 	victoryRazz(YELLOW);
								if (game.colourActive[BLUE] == TRUE) 	victoryRazz(BLUE);

								gameOver = TRUE;
							}
						}
					}
				}
			}
		}

		// Delay before sequence replay
		if (gameNumber == 1 || gameNumber == 3) delayMilliSeconds(800);
	}
}

// This function plays the victory razz
void victoryRazz(unsigned char colour)
{
	unsigned char counter;

	delayMilliSeconds(800);

	for (counter = 0; counter < 12; counter++)
	{
		lightOn(counter);
		delayMilliSeconds(80);
		lightOff(counter);
	}

	colourOn(BLUE);
	soundOn(BLUE_SOUND);
	delayMilliSeconds(400);
	soundOff();
	colourOff(BLUE);

	colourOn(YELLOW);
	soundOn(YELLOW_SOUND);
	delayMilliSeconds(400);
	soundOff();
	colourOff(YELLOW);

	colourOn(YELLOW);
	soundOn(YELLOW_SOUND);
	delayMilliSeconds(200);
	soundOff();
	colourOff(YELLOW);

	colourOn(YELLOW);
	soundOn(YELLOW_SOUND);
	delayMilliSeconds(100);
	soundOff();
	colourOff(YELLOW);

	colourOn(YELLOW);
	soundOn(YELLOW_SOUND);
	delayMilliSeconds(200);
	soundOff();
	colourOff(YELLOW);

	colourOn(RED);
	soundOn(RED_SOUND);
	delayMilliSeconds(200);
	soundOff();
	colourOff(RED);

	colourOn(GREEN);
	soundOn(GREEN_SOUND);
	delayMilliSeconds(200);
	soundOff();
	colourOff(GREEN);

	colourOn(RED);
	soundOn(RED_SOUND);
	delayMilliSeconds(200);
	soundOff();
	colourOff(RED);

	colourOn(YELLOW);
	soundOn(YELLOW_SOUND);
	delayMilliSeconds(400);
	soundOff();
	colourOff(YELLOW);

	colourOn(YELLOW);
	soundOn(YELLOW_SOUND);
	delayMilliSeconds(200);
	soundOff();
	colourOff(YELLOW);

	colourOn(YELLOW);
	soundOn(YELLOW_SOUND);
	delayMilliSeconds(100);
	soundOff();
	colourOff(YELLOW);

	colourOn(YELLOW);
	soundOn(YELLOW_SOUND);
	delayMilliSeconds(400);
	soundOff();
	colourOff(YELLOW);

	for (counter = 0; counter < 12; counter++)
	{
		lightOn(counter);
		delayMilliSeconds(80);
		lightOff(counter);
	}
}

// This function plays the special victory razz used when the player completes
// game 3 on skill level 4
void specialVictoryRazz(void)
{
	unsigned char loop;

	delayMilliSeconds(800);

	for (loop = 0; loop < 4; loop++)
	{
		colourOn(RED);
		gameSound(RED);
		if (loop == 0) delayMilliSeconds(50);
		else delayMilliSeconds(100);
		colourOff(RED);
		soundOff();

		colourOn(YELLOW);
		gameSound(YELLOW);
		delayMilliSeconds(100);
		colourOff(YELLOW);
		soundOff();

		if (loop < 3)
		{
			colourOn(BLUE);
			gameSound(BLUE);
			delayMilliSeconds(100);
			colourOff(BLUE);
			soundOff();

			colourOn(GREEN);
			gameSound(GREEN);
			delayMilliSeconds(100);
			colourOff(GREEN);
			soundOff();
		}
	}

	// Note: In the original game the FAIL sound would be played until the
	// game was turned off.  Here we play it for 2 seconds.  To make this 
	// emulate the original game properly replace the delay with an infinite
	// loop.
	gameSound(FAIL);
	delayMilliSeconds(2000); // while(1);
	soundOff();
}