/************************************************************************
	hardware.h

    Simon18F - An MB Electronics Simon clone based on the PIC18F2550
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef _HARDWARE_H_
#define _HARDWARE_H_

// Fosc frequency (48 Mhz)
#define _XTAL_FREQ 48000000

// Hardware mapping definitions:

// Serial chip select lines
#define CS_LED	RB5
#define CS_POT	RC1
#define OE_LED	RC2

// Identifiers for the 12 LEDs
#define GREENL		0
#define GREENC		1
#define GREENR		2

#define REDL		3
#define REDC		4
#define REDR		5

#define BLUEL		6
#define BLUEC		7
#define BLUER		8

#define YELLOWL		9
#define YELLOWC		10
#define YELLOWR		11

// Colour identifiers
#define GREEN	0
#define RED		1
#define YELLOW	2
#define BLUE	3
#define FAIL	4

// Speaker 
#define SPEAKER	RC0

// Outputs to buttons
#define START_SKILL			RB2
#define LAST_LONGEST_GAME	RB3
#define COLOUR_BUTTONS		RB4

// Inputs
#define IN1	RA0
#define IN2	RA1
#define IN3	RA2
#define IN4	RA3

// Other useful definitions
#define OFF		0
#define ON		1
#define FALSE	0
#define TRUE	1

// Tones for the colours (ADF#A - true tones :)
#define GREEN_SOUND		440
#define RED_SOUND		370
#define YELLOW_SOUND	294
#define BLUE_SOUND		220

// Tone for failure
#define FAIL_SOUND		 42

// Sequence playback speeds (in milliseconds)
#define PLAYBACK_SLOW	420
#define PLAYBACK_MEDIUM	320
#define PLAYBACK_FAST	220

// The following setting sets the speed of the timer0
// interrupt
#define TMR0_SPEED 0x00

// The debounce tolerance is the amount of readings to take
// of a button's state before changing it's state to either
// on or off.  The 'noisier' the buttons, the higher the 
// tolerance number should be.
#define DEBOUNCE_TOLERANCE 4

// Structure for game hardware state mapping:

// Push buttons including debounce variables
struct buttonState {
	unsigned char state; // On or off
	unsigned char debounceCount;
};

// Sliding switches
struct switchState {
	unsigned char position;
};

// Sound state
struct soundState {
	unsigned char state;
	unsigned char pwmState;
	unsigned char timer1LowByte;
	unsigned char timer1HighByte;
};

struct lightState {
	unsigned char state;
	int brightness;
	int dimSpeed;
};

// Overall hardware state
struct hardwareState {
	struct buttonState lastButton;
	struct buttonState startButton;
	struct buttonState longestButton;

	struct buttonState colourButton[4];
	struct lightState light[12];

	struct switchState gameSwitch;
	struct switchState skillSwitch;

	struct soundState sound;

	// Internal counters
	unsigned char pwmCounter;
	int dimCounter;
	unsigned char buttonCounter;

	// Delay counter
	unsigned long delayCounter;

	// Volume level
	int volumeLevel;
	int currentVolumeLevel;
} hardware;

// Game state
struct gameState {
	unsigned char gameActive;

	unsigned char gameNumber;
	unsigned char skillLevel;

	// For game 3, shows if a colour is active
	unsigned char colourActive[4];

	unsigned char currentSequence[32];
	int currentSequencePointer;
	int playerSequencePointer;

	unsigned char longestSequence[32];
	int longestSequenceLength;

	// The random number is generated by constantly changing
	// the variable between 0 and 3 using the timer0 interrupt
	unsigned char randomNumber;

	// The restart flag is used to indicate that the player
	// pressed the start key during a game.
	unsigned char restartFlag;
} game;

// Function prototypes
void initialise(void);

#endif