/************************************************************************
	main.c

    Simon18F - An MB Electronics Simon clone based on the PIC18F2550
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

// Global includes
#include <htc.h>

// Local includes
#include "hardware.h"
#include "interrupt.h"
#include "sound.h"
#include "delay.h"
#include "lights.h"
#include "game.h"

// PIC 18F2550 fuse configuration:
// Config word 1 (Oscillator configuration)
// 20Mhz crystal input scaled to 48Mhz and configured for USB operation
// from Table 2-3 page 30 of the datasheet
__CONFIG(1, USBPLL & IESODIS & FCMDIS & HSPLL & CPUDIV1 & PLLDIV5);
// Config word 2
__CONFIG(2, VREGEN & PWRTDIS & BORDIS & WDTDIS);
// Config word 3
__CONFIG(3, PBDIGITAL & LPT1DIS & MCLREN);
// Config word 4
__CONFIG(4, XINSTDIS & STVRDIS & LVPDIS & ICPORTDIS & DEBUGDIS);
// Config word 5, 6 and 7 (protection configuration)
__CONFIG(5, UNPROTECT);
__CONFIG(6, UNPROTECT);
__CONFIG(7, UNPROTECT);

// Simon's a computer, Simon has a brain, you either do what Simon says or else go down the drain
void main(void)
{
	// Turn off analogue ports
	ADCON1 = 0x0F;

	// Configure ports as inputs (1) or outputs(0)
	TRISA = 0b00001111;
	TRISB = 0b00000000;
	TRISC = 0b00000000;

	// Clear all ports
	PORTA = 0b00000000;
	PORTB = 0b00000000;
	PORTC = 0b00000000;

	// Set up SPI
	SSPSTAT = 0b11000000;	// SPI Master
							// Tx occurs on idle to active transition

	SSPCON1 = 0b00100000;	// Serial port enabled
							// Speed is Fosc/4

	// Initialise the state model
	initialise();

	// Enable interrupts with priority
	IPEN = 1;

	// Timer0 is used to generate an interrupt which reads the
	// state of the buttons and writes the state of the lights
	// in accordance to the state model.  Timer0 causes a low
	// priority interrupt so as not to disturb the higher
	// priority interrupt which is used to generate sound.
	//
	// The timer0 interrupt is also used as a delay counter
	// by the delay routines.
	TMR0IP = 0;			// Set timer0 interrupt to low priority
	TMR0IF = 0;			// Clear the timer0 interrupt flag
	TMR0L = TMR0_SPEED;	// Reset the timer0 counter
	T0CON = 0b11000000; // Timer0 on, 8-bit and 1:2 prescaler
	TMR0IE = 1;			// Enable the timer0 interrupt

	// Timer1 is used to generate a PWM signal for sound
	// Here we just initialise it ready for the sound routines to use
	// when a sound is requested.
	TMR1IP = 1;			// Set timer1 interrupt to high priority
	TMR1H = 0;			// Reset the timer1 counter (high byte)
	TMR1L = 0;			// Reset the timer1 counter (low byte)
	T1CON = 0b10100001;	// Timer1 on, 16-bit and 1:4 prescaler

	// Enable interrupts
	GIEH = 1;			// Global enable all high priority interrupts
	GIEL = 1;			// Global enable all low priority interrupts

	// Reset the delay counter for inactivity warning
	hardware.delayCounter = 0;

	// Enable the LED output
	OE_LED = 0;

	unsigned char counter, counter1;
	unsigned char warningColour;

	// Power on greeting (rotates the lights and then plays the
	// 5 notes from Close Encounters of the Third Kind :)
	for (counter = 0; counter < 12; counter++)
	{
		lightOn(counter);
		delayMilliSeconds(80);
		lightOff(counter);
	}
	for (counter = 0; counter < 12; counter++)
	{
		lightOn(counter);
		delayMilliSeconds(80);
		lightOff(counter);
	}
	delayMilliSeconds(200);

	colourOn(RED);
	soundOn(294);
	delayMilliSeconds(500);
	soundOff();
	colourOff(RED);

	colourOn(GREEN);
	soundOn(330);
	delayMilliSeconds(500);
	soundOff();
	colourOff(GREEN);

	colourOn(RED);
	soundOn(262);
	delayMilliSeconds(500);
	soundOff();
	colourOff(RED);

	colourOn(BLUE);
	soundOn(131);
	delayMilliSeconds(500);
	soundOff();
	colourOff(BLUE);

	colourOn(YELLOW);
	soundOn(196);
	delayMilliSeconds(1000);
	soundOff();	
	colourOff(YELLOW);

	delayMilliSeconds(1000);

	// Main game loop
	while(1)
	{
		mainGameLoop();

		// Check for inactivity longer than 45 seconds
		if (hardware.delayCounter > 720000)
		{
			// warn that the game is still on by spinning round the colours
			for (counter1 = 0; counter1 < 2; counter1++)
			{
				for (counter = 0; counter < 12; counter++)
				{
					lightOn(counter);
					delayMilliSeconds(80);
					lightOff(counter);
				}
			}

			// Reset the delay counter
			hardware.delayCounter = 0;
		}
	}
}